@@ -1,35 +1,35 @@
 /*
  * NMEA2000 over CAN.
  *
- * This file is Copyright (c) 2012 by the GPSD project
+ * This file is Copyright (c) 2012-2018 by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  */
 
-/* need this for strnlen() and struct ifreq */
-#define _DEFAULT_SOURCE
+#include "gpsd_config.h"  /* must be before all includes */
 
+#if defined(NMEA2000_ENABLE)
+
+#include <ctype.h>
+#include <fcntl.h>
+#include <linux/can.h>
+#include <linux/can/raw.h>
+#include <math.h>
+#include <net/if.h>
+#include <stdarg.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdbool.h>
 #include <string.h>
-#include <stdarg.h>
-#include <ctype.h>
-#include <time.h>
-#include <math.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <sys/socket.h>
 #include <sys/ioctl.h>
-#include <net/if.h>
+#include <sys/socket.h>
+#include <time.h>
+#include <unistd.h>
 
 #include "gpsd.h"
 #include "libgps.h"
-#if defined(NMEA2000_ENABLE)
 #include "driver_nmea2000.h"
 #include "bits.h"
 
-#include <linux/can.h>
-#include <linux/can/raw.h>
 
 #define LOG_FILE 1
 #define NMEA2000_NETS 4
@@ -1484,7 +1484,7 @@ static void find_pgn(struct can_frame *frame, struct gps_device_t *session)
 	} else {
 	    // we got a unknown unit number
 	    if (nmea2000_units[can_net][source_unit] == NULL) {
-	        char buffer[32];
+	        char buffer[55];
 
 		(void) snprintf(buffer,
 				sizeof(buffer),
@@ -1539,7 +1539,7 @@ static gps_mask_t nmea2000_parse_input(struct gps_device_t *session)
 
 int nmea2000_open(struct gps_device_t *session)
 {
-    char interface_name[strlen(session->gpsdata.dev.path)+1];
+    char interface_name[GPS_PATH_MAX];
     socket_t sock;
     int status;
     int unit_number;
@@ -1675,7 +1675,7 @@ int nmea2000_open(struct gps_device_t *session)
 	}
     }
 
-    session->gpsdata.dev.parity = 'n';
+    session->gpsdata.dev.parity = 'N';
     session->gpsdata.dev.baudrate = 250000;
     session->gpsdata.dev.stopbits = 0;
     return session->gpsdata.gps_fd;
@@ -1737,4 +1737,8 @@ const struct gps_type_t driver_nmea2000 = {
 
 /* end */
 
+#else   /* of  defined(NMEA2000_ENABLE) */
+/* dummy variable to some old linkers do not commplain about empty
+ * object file */
+int nmea2000_dummy = 1;
 #endif /* of  defined(NMEA2000_ENABLE) */
