@@ -32,9 +32,12 @@
  * packet.c
  * packet_states.h
  *
- * This file is Copyright (c) 2010 by the GPSD project
- * BSD terms apply: see the file COPYING in the distribution root for details.
+ * This file is Copyright (c) 2010-2018 by the GPSD project
+ * SPDX-License-Identifier: BSD-2-clause
  */
+
+#include "gpsd_config.h"  /* must be before all includes */
+
 #include <stdio.h>
 #include <stdbool.h>
 #include <string.h>
@@ -83,10 +86,18 @@ _proto__msg_navsol(struct gps_device_t *session, unsigned char *buf, size_t data
 
     /* extract ECEF navigation solution here */
     /* or extract the local tangential plane (ENU) solution */
-    [Px, Py, Pz, Vx, Vy, Vz] = GET_ECEF_FIX();
+    [session->newdata.ecef.x,
+    session->newdata.ecef.y,
+    session->newdata.ecef.z,
+    session->newdata.ecef.vx,
+    session->newdata.ecef.vy,
+    session->newdata.ecef.vz] = GET_ECEF_FIX();
     ecef_to_wgs84fix(&session->newdata,  &session->gpsdata.separation,
-		     Px, Py, Pz, Vx, Vy, Vz);
-    mask |= LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET  ;
+		     session->newdata.ecef.x, session->newdata.ecef.y,
+		     session->newdata.ecef.z, session->newdata.ecef.vx,
+		     session->newdata.ecef.vy, session->newdata.ecef.vz);
+    mask |= LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET
+            | ECEF_SET | VECEF_SET  ;
 
     session->newdata.epx = GET_LONGITUDE_ERROR();
     session->newdata.epy = GET_LATITUDE_ERROR();
@@ -227,6 +238,7 @@ _proto__msg_raw(struct gps_device_t *session, unsigned char *buf, size_t data_le
 {
     unsigned char i, st, nchan, nsv;
     unsigned int tow;
+    double t_intp;
 
     if (data_len != RAW_MSG_LEN )
 	return 0;
@@ -252,16 +264,22 @@ _proto__msg_raw(struct gps_device_t *session, unsigned char *buf, size_t data_le
 	return 0;
     }
 
+    session->gpsdata.raw.mtime.tv_nsec =
+        modf(session->newdata.time, &t_intp) * 10e8;
+    session->gpsdata.raw.mtime.tv_sec = (time_t)t_intp;
+
+    /* this is so we can tell which never got set */
+    for (i = 0; i < MAXCHANNELS; i++)
+        session->gpsdata.raw.meas[i].svid = 0;
     for (i = 0; i < n; i++){
 	session->gpsdata.PRN[i] = GET_PRN();
 	session->gpsdata.ss[i] = GET_SIGNAL()
-	session->gpsdata.raw.satstat[i] = GET_FLAGS();
-	session->gpsdata.raw.pseudorange[i] = GET_PSEUDORANGE();
-	session->gpsdata.raw.doppler[i] = GET_DOPPLER();
-	session->gpsdata.raw.carrierphase[i] = GET_CARRIER_PHASE();
-	session->gpsdata.raw.mtime[i] = GET_MEASUREMENT_TIME();
-	session->gpsdata.raw.codephase[i] = GET_CODE_PHASE();
-	session->gpsdata.raw.deltarange[i] = GET_DELTA_RANGE();
+	session->gpsdata.raw.meas[i].satstat = GET_FLAGS();
+	session->gpsdata.raw.meas[i].pseudorange = GET_PSEUDORANGE();
+	session->gpsdata.raw.meas[i].doppler = GET_DOPPLER();
+	session->gpsdata.raw.meas[i].carrierphase = GET_CARRIER_PHASE();
+	session->gpsdata.raw.meas[i].codephase = GET_CODE_PHASE();
+	session->gpsdata.raw.meas[i].deltarange = GET_DELTA_RANGE();
     }
     return RAW_IS;
 }
