@@ -1,11 +1,13 @@
 /*
- * This file is Copyright (c) 2010 by the GPSD project
- * BSD terms apply: see the file COPYING in the distribution root for details.
+ * This file is Copyright (c) 2010-2018 by the GPSD project
+ * SPDX-License-Identifier: BSD-2-clause
  */
 
 /* for vsnprintf() FreeBSD wants __ISO_C_VISIBLE >= 1999 */
 #define __ISO_C_VISIBLE 1999
 
+#include "gpsd_config.h"  /* must be before all includes */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -46,7 +48,7 @@ gps_mask_t generic_parse_input(struct gps_device_t *session)
 
 	if ((st=nmea_parse(sentence, session)) == 0) {
 	    gpsd_log(&session->context->errout, LOG_WARN,
-		     "unknown sentence: \"%s\"\n",	sentence);
+		     "unknown sentence: \"%s\"\n", sentence);
 	}
 	for (dp = gpsd_drivers; *dp; dp++) {
 	    char *trigger = (*dp)->trigger;
@@ -242,6 +244,15 @@ static void nmea_event_hook(struct gps_device_t *session, event_t event)
 	    (void)nmea_send(session, "$PMTK605");
 	    break;
 #endif /* MTK3301_ENABLE */
+#ifdef GREIS_ENABLE
+	case 9:
+	    /* probe for Javad GREIS -- expect reply with JAVAD */
+	    gpsd_log(&session->context->errout, LOG_PROG,
+		     "=> Probing for Javad GREIS\n");
+	    /* TODO: confirm this actually gets JAVAD response */
+	    (void)nmea_send(session, "print,/par/rcv/vendor");
+	    break;
+#endif /* GREIS_ENABLE */
 	default:
 	    break;
 	}
@@ -1376,6 +1387,7 @@ static bool aivdm_decode(const char *buf, size_t buflen,
     unsigned char *field[NMEA_MAX*2];
     unsigned char fieldcopy[NMEA_MAX*2+1];
     unsigned char *data, *cp;
+    char const *cp1;
     int pad;
     struct aivdm_context_t *ais_context;
     int i;
@@ -1397,6 +1409,15 @@ static bool aivdm_decode(const char *buf, size_t buflen,
 	return false;
     }
 
+    /* discard sentences with high-half characters in them, they're corrupted */
+    for (cp1 = buf; *cp1; cp1++) {
+	if (!isascii(*cp1)) {
+	    gpsd_log(&session->context->errout, LOG_ERROR,
+		     "corrupted AIVDM packet.\n");
+	    return false;
+	}
+    }
+
     /* extract packet fields */
     (void)strlcpy((char *)fieldcopy, buf, sizeof(fieldcopy));
     field[nfields++] = (unsigned char *)buf;
@@ -1741,6 +1762,7 @@ extern const struct gps_type_t driver_evermore;
 extern const struct gps_type_t driver_garmin_ser_binary;
 extern const struct gps_type_t driver_garmin_usb_binary;
 extern const struct gps_type_t driver_geostar;
+extern const struct gps_type_t driver_greis;
 extern const struct gps_type_t driver_italk;
 extern const struct gps_type_t driver_navcom;
 extern const struct gps_type_t driver_nmea2000;
@@ -1803,6 +1825,9 @@ static const struct gps_type_t *gpsd_driver_array[] = {
 #ifdef GEOSTAR_ENABLE
     &driver_geostar,
 #endif /* GEOSTAR_ENABLE */
+#ifdef GREIS_ENABLE
+    &driver_greis,
+#endif /* GREIS_ENABLE */
 #ifdef ITRAX_ENABLE
     &driver_italk,
 #endif /* ITRAX_ENABLE */
