@@ -1,12 +1,23 @@
 /*
- * This file is Copyright (c) 2010 by the GPSD project
+ * Python binding for selected libgps library functions
+ *
+ * This file is Copyright (c) 2010-2018 by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  *
- * Python binding for selected libgps library functions
  */
+
+#include "gpsd_config.h"  /* must be before all includes */
+
+/* Python.h insists on setting GNU_SOURCE, _POSIX_C_SOURCE and
+ * _XOPEN_SOURCE.  Without guards. */
+#undef _GNU_SOURCE
+#undef _POSIX_C_SOURCE
+#undef _XOPEN_SOURCE
+
 #include <Python.h>
 
 #include <stdio.h>
+
 #include "gps.h"
 #include "gpsdclient.h"
 #include "compiler.h"	/* for UNUSED */
