@@ -1,8 +1,10 @@
 /*
- * This file is Copyright (c) 2010 by the GPSD project
+ * This file is Copyright (c) 2010-2018 by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  */
 
+#include "gpsd_config.h"  /* must be before all includes */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -16,7 +18,6 @@
 #include <unistd.h>
 
 #include "gps.h"
-#include "gpsd_config.h"
 #include "gpsdclient.h"
 #include "revision.h"
 #include "os_compat.h"
@@ -45,7 +46,7 @@ static void print_gpx_header(void)
 
     (void)fprintf(logfile,"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
     (void)fprintf(logfile,"<gpx version=\"1.1\" creator=\"GPSD %s - %s\"\n", VERSION, GPSD_URL);
-    (void)fprintf(logfile,"        xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
+    (void)fprintf(logfile,"        xmlns:xsi=\"https://www.w3.org/2001/XMLSchema-instance\"\n");
     (void)fprintf(logfile,"        xmlns=\"http://www.topografix.com/GPX/1/1\"\n");
     (void)fprintf(logfile,"        xsi:schemaLocation=\"http://www.topografix.com/GPX/1/1\n");
     (void)fprintf(logfile,"        http://www.topografix.com/GPX/1/1/gpx.xsd\">\n");
