@@ -97,7 +97,7 @@ else:  # Otherwise we do something real
 
 
 def isotime(s):
-    """Convert timestamps in ISO8661 format to and from Unix time including
+    """Convert timestamps in ISO8601 format to and from Unix time including
     optional fractional seconds.
     """
 
@@ -272,11 +272,15 @@ def make_leapsecond_include(infile):
  * Constants used for GPS time detection and rollover correction.
  *
  * Correct for week beginning %(_isodate)s
+ *
+ * Autogenerated, do not edit. All changes will be undone.
  */
 #define BUILD_CENTURY\t%(_century)s
 #define BUILD_WEEK\t%(_week)d                   # Assumes 10-bit week counter
 #define BUILD_LEAPSECONDS\t%(_leapsecs)d
 #define BUILD_ROLLOVERS\t%(_rollovers)d         # Assumes 10-bit week counter
+
+/* Autogenerated, do not edit. All changes will be undone. */
 """ % locals()
 
 
