@@ -1,25 +1,24 @@
 /* libgps_sock.c -- client interface library for the gpsd daemon
  *
- * This file is Copyright (c) 2010 by the GPSD project
- * BSD terms apply: see the file COPYING in the distribution root for details.
+ * This file is Copyright (c) 2010-2018 by the GPSD project
+ * SPDX-License-Identifier: BSD-2-clause
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdbool.h>
-#include <fcntl.h>
-#include <string.h>
+
+#include "gpsd_config.h"  /* must be before all includes */
+
+#include <assert.h>
 #include <ctype.h>
 #include <errno.h>
-#include <math.h>
+#include <fcntl.h>
 #include <locale.h>
-#include <assert.h>
-#include <sys/time.h>	 /* expected to have a select(2) prototype a la SuS */
-#include <sys/types.h>
-#include <sys/stat.h>
-#include "gpsd_config.h"
-#ifdef HAVE_SYS_SELECT_H
+#include <math.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/select.h>
-#endif /* HAVE_SYS_SELECT_H */
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <unistd.h>
 
 #ifndef USE_QT
@@ -37,6 +36,7 @@
 #include "gpsd.h"
 #include "libgps.h"
 #include "strfuncs.h"
+#include "timespec.h"      /* for NS_IN_SEC */
 #ifdef SOCKET_EXPORT_ENABLE
 #include "gps_json.h"
 
@@ -131,22 +131,12 @@ bool gps_sock_waiting(const struct gps_data_t *gpsdata, int timeout)
 /* timeout is in uSec */
 {
 #ifndef USE_QT
-    fd_set rfds;
-    struct timeval tv;
-
     libgps_debug_trace((DEBUG_CALLS, "gps_waiting(%d): %d\n", timeout, PRIVATE(gpsdata)->waitcount++));
     if (PRIVATE(gpsdata)->waiting > 0)
 	return true;
 
-    /* we might want to check for EINTR if this returns false */
-    errno = 0;
-
-    FD_ZERO(&rfds);
-    FD_SET(gpsdata->gps_fd, &rfds);
-    tv.tv_sec = timeout / 1000000;
-    tv.tv_usec = timeout % 1000000;
     /* all error conditions return "not waiting" -- crude but effective */
-    return (select(gpsdata->gps_fd + 1, &rfds, NULL, NULL, &tv) == 1);
+    return nanowait(gpsdata->gps_fd, timeout * 1000);
 #else
     return ((QTcpSocket *) (gpsdata->gps_fd))->waitForReadyRead(timeout / 1000);
 #endif
@@ -178,23 +168,27 @@ int gps_sock_close(struct gps_data_t *gpsdata)
 #endif
 }
 
-int gps_sock_read(struct gps_data_t *gpsdata)
+int gps_sock_read(struct gps_data_t *gpsdata, char *message, int message_len)
 /* wait for and read data being streamed from the daemon */
 {
     char *eol;
     ssize_t response_length;
     int status = -1;
 
-    gpsdata->set &= ~PACKET_SET;
-    for (eol = PRIVATE(gpsdata)->buffer;
-	 *eol != '\n' && eol < PRIVATE(gpsdata)->buffer + PRIVATE(gpsdata)->waiting; eol++)
-	continue;
-    if (*eol != '\n')
-	eol = NULL;
-
     errno = 0;
+    gpsdata->set &= ~PACKET_SET;
+
+    /* scan to find end of message (\n), or end of buffer */
+    for (eol = PRIVATE(gpsdata)->buffer;
+	 eol < (PRIVATE(gpsdata)->buffer + PRIVATE(gpsdata)->waiting);
+         eol++) {
+        if ('\n' == *eol)
+            break;
+    }
+
+    if (*eol != '\n') {
+	/* no full message found, try to fill buffer */
 
-    if (eol == NULL) {
 #ifndef USE_QT
 	/* read data: return -1 if no data waiting or buffered, 0 otherwise */
 	status = (int)recv(gpsdata->gps_fd,
@@ -210,11 +204,13 @@ int gps_sock_read(struct gps_data_t *gpsdata)
 #ifdef HAVE_WINSOCK2_H
 	int wserr = WSAGetLastError();
 #endif /* HAVE_WINSOCK2_H */
+
 	/* if we just received data from the socket, it's in the buffer */
 	if (status > -1)
 	    PRIVATE(gpsdata)->waiting += status;
-	/* buffer is empty - implies no data was read */
+
 	if (PRIVATE(gpsdata)->waiting == 0) {
+	    /* buffer is empty - implies no data was read */
 	    /*
 	     * If we received 0 bytes, other side of socket is closing.
 	     * Return -1 as end-of-data indication.
@@ -237,24 +233,43 @@ int gps_sock_read(struct gps_data_t *gpsdata)
 	    else
 		return -1;
 	}
-	/* there's buffered data waiting to be returned */
+
+	/* there's new buffered data waiting, check for full message */
 	for (eol = PRIVATE(gpsdata)->buffer;
-	     *eol != '\n' && eol < PRIVATE(gpsdata)->buffer + PRIVATE(gpsdata)->waiting; eol++)
-	    continue;
+	     eol < (PRIVATE(gpsdata)->buffer + PRIVATE(gpsdata)->waiting);
+             eol++) {
+	    if ('\n' == *eol)
+		break;
+	}
 	if (*eol != '\n')
-	    eol = NULL;
-	if (eol == NULL)
+            /* still no full message, give up for now */
 	    return 0;
     }
 
-    assert(eol != NULL);
+    /* eol now points to trailing \n in a full message */
     *eol = '\0';
-    response_length = eol - PRIVATE(gpsdata)->buffer + 1;
+    if (NULL != message) {
+        strlcpy(message, PRIVATE(gpsdata)->buffer, message_len);
+    }
     gpsdata->online = timestamp();
+    /* unpack the JSON message */
     status = gps_unpack(PRIVATE(gpsdata)->buffer, gpsdata);
-    memmove(PRIVATE(gpsdata)->buffer,
-	    PRIVATE(gpsdata)->buffer + response_length, PRIVATE(gpsdata)->waiting - response_length);
+
+    /* why the 1? */
+    response_length = eol - PRIVATE(gpsdata)->buffer + 1;
+
+    /* calculate length of good data still in buffer */
     PRIVATE(gpsdata)->waiting -= response_length;
+
+    if (1 > PRIVATE(gpsdata)->waiting) {
+        /* no waiting data, or overflow, clear the buffer, just in case */
+        *PRIVATE(gpsdata)->buffer = '\0';
+        PRIVATE(gpsdata)->waiting = 0;
+    } else {
+	memmove(PRIVATE(gpsdata)->buffer,
+		PRIVATE(gpsdata)->buffer + response_length,
+		PRIVATE(gpsdata)->waiting);
+    }
     gpsdata->set |= PACKET_SET;
 
     return (status == 0) ? (int)response_length : status;
@@ -292,6 +307,7 @@ int gps_unpack(char *buf, struct gps_data_t *gpsdata)
 const char *gps_sock_data(const struct gps_data_t *gpsdata)
 /* return the contents of the client data buffer */
 {
+    /* no length data, so pretty useless... */
     return PRIVATE(gpsdata)->buffer;
 }
 
@@ -384,7 +400,7 @@ int gps_sock_mainloop(struct gps_data_t *gpsdata, int timeout,
 	if (!gps_waiting(gpsdata, timeout)) {
 	    return -1;
 	} else {
-	    int status = gps_read(gpsdata);
+	    int status = gps_read(gpsdata, NULL, 0);
 
 	    if (status == -1)
 		return -1;
