@@ -1,8 +1,11 @@
 /* net_dgpsip.c -- gather and dispatch DGPS data from DGPSIP servers
  *
- * This file is Copyright (c) 2010 by the GPSD project
- * BSD terms apply: see the file COPYING in the distribution root for details.
+ * This file is Copyright (c) 2010-2018 by the GPSD project
+ * SPDX-License-Identifier: BSD-2-clause
  */
+
+#include "gpsd_config.h"  /* must be before all includes */
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdbool.h>
