@@ -1,7 +1,10 @@
 /*
- * This file is Copyright (c) 2010 by the GPSD project
+ * This file is Copyright (c) 2010-2018 by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  */
+
+#include "gpsd_config.h"  /* must be before all includes */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -21,31 +24,80 @@
  * value NAN, it is a valid WGS84 geoidal separation in meters for the fix.
  */
 
-static double degtodm(double angle)
+#define BUF_SZ 20
+static char *degtodm_str(double angle, const char *fmt, char *buf)
 /* decimal degrees to GPS-style, degrees first followed by minutes */
 {
     double fraction, integer;
-    fraction = modf(angle, &integer);
-    return floor(angle) * 100 + fraction * 60;
+    if (0 == isfinite(angle)) {
+        buf[0] = '\0';
+    } else {
+        angle = fabs(angle);
+	fraction = modf(angle, &integer);
+	(void)snprintf(buf, BUF_SZ, fmt, floor(angle) * 100 + fraction * 60);
+    }
+    return buf;
 }
 
+/* format a float/lon/alt into a string, handle NAN, INFINITE */
+static char *f_str(double f, const char *fmt, char *buf)
+{
+    if (0 == isfinite(f)) {
+        buf[0] = '\0';
+    } else {
+	(void)snprintf(buf, BUF_SZ, fmt, f);
+    }
+    return buf;
+}
+
+/* convert UTC to time str (hh:mm:ss) and tm */
+static void utc_to_hhmmss(timestamp_t time, char *buf, ssize_t buf_sz,
+                          struct tm *tm)
+{
+    double integral;
+    double fractional;
+    time_t integral_time;
+    unsigned frac;
+
+    if (0 == isfinite(time)) {
+        buf[0] = '\0';
+        return;
+    }
+
+    /* round to 100ths */
+    fractional = modf(time + 0.005, &integral);
+    integral_time = (time_t)integral;
+    frac = (unsigned)(fractional * 100.0);
+
+    (void)gmtime_r(&integral_time, tm);
+
+    (void)snprintf(buf, buf_sz, "%02d%02d%02d.%02d",
+		   tm->tm_hour, tm->tm_min, tm->tm_sec, frac);
+    return;
+}
+
+/* Dump a $GPGGA.
+ * looks like this is only called from net_ntrip.c
+ */
 void gpsd_position_fix_dump(struct gps_device_t *session,
 			    char bufp[], size_t len)
 {
     struct tm tm;
-    time_t intfixtime;
+    char time_str[20];
+    char lat_str[BUF_SZ];
+    char lon_str[BUF_SZ];
+
+    utc_to_hhmmss(session->gpsdata.fix.time, time_str, sizeof(time_str), &tm);
 
-    intfixtime = (time_t) session->gpsdata.fix.time;
-    (void)gmtime_r(&intfixtime, &tm);
     if (session->gpsdata.fix.mode > MODE_NO_FIX) {
 	(void)snprintf(bufp, len,
-		       "$GPGGA,%02d%02d%02d,%09.4f,%c,%010.4f,%c,%d,%02d,",
-		       tm.tm_hour,
-		       tm.tm_min,
-		       tm.tm_sec,
-		       degtodm(fabs(session->gpsdata.fix.latitude)),
+		       "$GPGGA,%s,%s,%c,%s,%c,%d,%02d,",
+		       time_str,
+		       degtodm_str(session->gpsdata.fix.latitude, "%09.4f",
+                                   lat_str),
 		       ((session->gpsdata.fix.latitude > 0) ? 'N' : 'S'),
-		       degtodm(fabs(session->gpsdata.fix.longitude)),
+		       degtodm_str(session->gpsdata.fix.longitude, "%010.4f",
+                                   lon_str),
 		       ((session->gpsdata.fix.longitude > 0) ? 'E' : 'W'),
 		       session->gpsdata.status,
 		       session->gpsdata.satellites_used);
@@ -75,67 +127,89 @@ void gpsd_position_fix_dump(struct gps_device_t *session,
 static void gpsd_transit_fix_dump(struct gps_device_t *session,
 				  char bufp[], size_t len)
 {
+    char time_str[20];
+    char time2_str[20];
+    char lat_str[BUF_SZ];
+    char lon_str[BUF_SZ];
+    char speed_str[BUF_SZ];
+    char track_str[BUF_SZ];
     struct tm tm;
-    time_t intfixtime;
 
-    tm.tm_mday = tm.tm_mon = tm.tm_year = tm.tm_hour = tm.tm_min = tm.tm_sec =
-	0;
-    if (0 != isfinite(session->gpsdata.fix.time)) {
-	intfixtime = (time_t) session->gpsdata.fix.time;
-	(void)gmtime_r(&intfixtime, &tm);
+    utc_to_hhmmss(session->gpsdata.fix.time, time_str, sizeof(time_str), &tm);
+    if ('\0' != time_str[0]) {
 	tm.tm_mon++;
 	tm.tm_year %= 100;
+
+	(void)snprintf(time2_str, sizeof(time2_str),
+		       "%02d%02d%02d",
+		       tm.tm_mday, tm.tm_mon, tm.tm_year);
+    } else {
+        time2_str[0] = '\0';
     }
-#define ZEROIZE(x)	(isfinite(x)==0 ? 0.0 : x)
     (void)snprintf(bufp, len,
-		   "$GPRMC,%02d%02d%02d,%c,%09.4f,%c,%010.4f,%c,%.4f,%.3f,%02d%02d%02d,,",
-		   tm.tm_hour,
-		   tm.tm_min,
-		   tm.tm_sec,
+		   "$GPRMC,%s,%c,%s,%c,%s,%c,%s,%s,%s,,",
+		   time_str,
 		   session->gpsdata.status ? 'A' : 'V',
-		   ZEROIZE(degtodm(fabs(session->gpsdata.fix.latitude))),
+		   degtodm_str(session->gpsdata.fix.latitude, "%09.4f",
+                               lat_str),
 		   ((session->gpsdata.fix.latitude > 0) ? 'N' : 'S'),
-		   ZEROIZE(degtodm(fabs(session->gpsdata.fix.longitude))),
+		   degtodm_str(session->gpsdata.fix.longitude, "%010.4f",
+                               lon_str),
 		   ((session->gpsdata.fix.longitude > 0) ? 'E' : 'W'),
-		   ZEROIZE(session->gpsdata.fix.speed * MPS_TO_KNOTS),
-		   ZEROIZE(session->gpsdata.fix.track),
-		   tm.tm_mday, tm.tm_mon, tm.tm_year);
-#undef ZEROIZE
+		   f_str(session->gpsdata.fix.speed * MPS_TO_KNOTS, "%.4f",
+                            speed_str),
+		   f_str(session->gpsdata.fix.track, "%.3f", track_str),
+		   time2_str);
     nmea_add_checksum(bufp);
 }
 
 static void gpsd_binary_satellite_dump(struct gps_device_t *session,
 				       char bufp[], size_t len)
 {
-    int i;
+    int i;                  /* index into skyview[] */
+    int j;                  /* index into GPGSV */
     char *bufp2 = bufp;
+    int satellites_visible = 0;
     bufp[0] = '\0';
 
+    /* check skyview[] for valid sats first */
     for (i = 0; i < session->gpsdata.satellites_visible; i++) {
-	if (i % 4 == 0) {
+	if ( 1 > session->gpsdata.skyview[i].PRN) {
+	    /* bad prn, ignore */
+            continue;
+	}
+	if (90 < abs(session->gpsdata.skyview[i].elevation)) {
+	    /* bad elevation, ignore */
+            continue;
+	}
+	satellites_visible++;
+    }
+    for (i = 0, j= 0; i < session->gpsdata.satellites_visible; i++) {
+	if ( 1 > session->gpsdata.skyview[i].PRN) {
+	    /* bad prn, skip */
+	    continue;
+	}
+	if (90 < abs(session->gpsdata.skyview[i].elevation)) {
+	    /* bad elevation, ignore */
+            continue;
+	}
+	if (j % 4 == 0) {
 	    bufp2 = bufp + strlen(bufp);
 	    str_appendf(bufp, len,
 			    "$GPGSV,%d,%d,%02d",
-			    ((session->gpsdata.satellites_visible - 1) / 4) +
-			    1, (i / 4) + 1,
-			    session->gpsdata.satellites_visible);
+			    ((satellites_visible - 1) / 4) + 1, (j / 4) + 1,
+			    satellites_visible);
 	}
-	if (i < session->gpsdata.satellites_visible) {
-            if ( 0 == session->gpsdata.skyview[i].PRN) {
-                /* bad prn, just make all zeros */
-                str_appendf(bufp, len, ",00,00,000,00");
-            } else {
-                str_appendf(bufp, len,
-			    ",%02d,%02d,%03d,%02.0f",
-			    session->gpsdata.skyview[i].PRN,
-			    session->gpsdata.skyview[i].elevation,
-			    session->gpsdata.skyview[i].azimuth,
-			    session->gpsdata.skyview[i].ss);
-            }
-        }
-	if (i % 4 == 3 || i == session->gpsdata.satellites_visible - 1) {
+	str_appendf(bufp, len, ",%02d,%02d,%03d,%02.0f",
+		    session->gpsdata.skyview[i].PRN,
+		    session->gpsdata.skyview[i].elevation,
+		    session->gpsdata.skyview[i].azimuth,
+		    session->gpsdata.skyview[i].ss);
+
+	if (j % 4 == 3 || j == satellites_visible - 1) {
 	    nmea_add_checksum(bufp2);
 	}
+        j++;
     }
 
 #ifdef ZODIAC_ENABLE
@@ -206,56 +280,49 @@ static void gpsd_binary_quality_dump(struct gps_device_t *session,
 	}
 	nmea_add_checksum(bufp2);
     }
-    if (isfinite(session->gpsdata.fix.epx) != 0
-	&& isfinite(session->gpsdata.fix.epy) != 0
-	&& isfinite(session->gpsdata.fix.epv) != 0
-	&& isfinite(session->gpsdata.epe) != 0) {
-	struct tm tm;
-	time_t intfixtime;
 
-	tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
-	if (0 != isfinite(session->gpsdata.fix.time)) {
-	    intfixtime = (time_t) session->gpsdata.fix.time;
-	    (void)gmtime_r(&intfixtime, &tm);
-	}
+    /* create $GPGBS if we have time, epx and epy.  Optional epv.
+     * Not really kosher, not have enough info to compute the RAIM
+     */
+    if (0 != isfinite(session->gpsdata.fix.epx) &&
+	0 != isfinite(session->gpsdata.fix.epy) &&
+	0 != isfinite(session->gpsdata.fix.time)) {
+
+        struct tm tm;
+	char time_str[20];
+	char epv_str[BUF_SZ];
+
+	(void)utc_to_hhmmss(session->gpsdata.fix.time,
+                            time_str, sizeof(time_str), &tm);
+
 	bufp2 = bufp + strlen(bufp);
-#define ZEROIZE(x)      (isfinite(x)==0 ? 0.0 : x)
 	str_appendf(bufp, len,
-		       "$GPGBS,%02d%02d%02d,%.2f,M,%.2f,M,%.2f,M",
-		       tm.tm_hour, tm.tm_min, tm.tm_sec,
-		       ZEROIZE(session->gpsdata.fix.epx),
-		       ZEROIZE(session->gpsdata.fix.epy),
-		       ZEROIZE(session->gpsdata.fix.epv));
+		       "$GPGBS,%s,%.3f,%.3f,%s,,,,",
+		       time_str,
+		       session->gpsdata.fix.epx,
+		       session->gpsdata.fix.epy,
+		       f_str(session->gpsdata.fix.epv, "%.3f", epv_str));
 	nmea_add_checksum(bufp2);
     }
-#undef ZEROIZE
 }
 
+/* Dump $GPZDA if we have time and a fix */
 static void gpsd_binary_time_dump(struct gps_device_t *session,
 				     char bufp[], size_t len)
 {
-    struct tm tm;
-    double integral;
-    time_t integral_time;
 
-    if (session->newdata.mode > MODE_NO_FIX) {
-	double fractional = modf(session->newdata.time, &integral);
-	integral_time = (time_t) integral;
-	(void)gmtime_r(&integral_time, &tm);
-	/*
-	 * We pin this report to the GMT/UTC timezone.  This may be technically
-	 * incorrect; our sources on ZDA suggest that it should report local
-	 * timezone. But no GPS we've ever seen actually does this, because it
-	 * would require embedding a location-to-TZ database in the receiver.
-	 * And even if we could do that, it would make our regression tests
-	 * break any time they were run in a timezone different from the one
-	 * where they were generated.
-	 */
+    if (MODE_NO_FIX < session->newdata.mode &&
+        0 != isfinite(session->gpsdata.fix.time)) {
+	struct tm tm;
+	char time_str[20];
+
+	utc_to_hhmmss(session->gpsdata.fix.time, time_str, sizeof(time_str),
+                      &tm);
+        /* There used to be confusion, but we now know NMEA times are UTC,
+         * when available. */
 	(void)snprintf(bufp, len,
-		       "$GPZDA,%02d%02d%05.2f,%02d,%02d,%04d,00,00",
-		       tm.tm_hour,
-		       tm.tm_min,
-		       (double)tm.tm_sec + fractional,
+		       "$GPZDA,%s,%02d,%02d,%04d,00,00",
+		       time_str,
 		       tm.tm_mday,
 		       tm.tm_mon + 1,
 		       tm.tm_year + 1900);
@@ -268,20 +335,21 @@ static void gpsd_binary_almanac_dump(struct gps_device_t *session,
 {
     if ( session->gpsdata.subframe.is_almanac ) {
 	(void)snprintf(bufp, len,
-			"$GPALM,1,1,%02d,%04d,%02x,%04x,%02x,%04x,%04x,%05x,%06x,%06x,%06x,%03x,%03x",
-		       (int)session->gpsdata.subframe.sub5.almanac.sv,
-		       (int)session->context->gps_week % 1024,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.svh,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.e,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.toa,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.deltai,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.Omegad,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.sqrtA,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.omega,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.Omega0,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.M0,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.af0,
-		       (unsigned int)session->gpsdata.subframe.sub5.almanac.af1);
+	    "$GPALM,1,1,%02d,%04d,%02x,%04x,%02x,%04x,%04x,%05x,"
+            "%06x,%06x,%06x,%03x,%03x",
+	    (int)session->gpsdata.subframe.sub5.almanac.sv,
+	    (int)session->context->gps_week % 1024,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.svh,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.e,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.toa,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.deltai,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.Omegad,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.sqrtA,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.omega,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.Omega0,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.M0,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.af0,
+	    (unsigned int)session->gpsdata.subframe.sub5.almanac.af1);
 	nmea_add_checksum(bufp);
     }
 }
@@ -410,7 +478,7 @@ void nmea_tpv_dump(struct gps_device_t *session,
 			      len - strlen(bufp));
     }
     if ((session->gpsdata.set
-	 & (MODE_SET | DOP_SET | USED_IS | HERR_SET | VERR_SET)) != 0)
+	 & (MODE_SET | DOP_SET | USED_IS | HERR_SET)) != 0)
 	gpsd_binary_quality_dump(session, bufp + strlen(bufp),
 				 len - strlen(bufp));
 }
