@@ -10,11 +10,13 @@ the daemon and the client library.
 
 PERMISSIONS
   Written by Eric S. Raymond, 2009
-  This file is Copyright (c) 2010 by the GPSD project
-  BSD terms apply: see the file COPYING in the distribution root for details.
+  This file is Copyright (c) 2009-2018 The GPSD project
+  SPDX-License-Identifier: BSD-2-clause
 
 ***************************************************************************/
 
+#include "gpsd_config.h"  /* must be before all includes */
+
 #include <math.h>
 #include <stdbool.h>
 
@@ -41,6 +43,8 @@ int json_device_read(const char *buf,
 	                                .len = sizeof(dev->driver)},
 	{"subtype",    t_string,     .addr.string  = dev->subtype,
 	                                .len = sizeof(dev->subtype)},
+        {"hexdata",    t_string,     .addr.string  = dev->hexdata,
+	                                .len = sizeof(dev->hexdata)},
 	{"native",     t_integer,    .addr.integer = &dev->driver_mode,
 				        .dflt.integer = DEVDEFAULT_NATIVE},
 	{"bps",	       t_uinteger,   .addr.uinteger = &dev->baudrate,
@@ -63,7 +67,7 @@ int json_device_read(const char *buf,
     if (status != 0)
 	return status;
 
-    if (isnan(dev->activated)!=0) {
+    if (isfinite(dev->activated) == 0) {
 	if (tbuf[0] == '\0')
 	    dev->activated = NAN;
 	else
@@ -74,7 +78,7 @@ int json_device_read(const char *buf,
 }
 
 int json_watch_read(const char *buf,
-		    struct policy_t *ccp,
+		    struct gps_policy_t *ccp,
 		    const char **endptr)
 {
     bool dummy_pps_flag;
